#ifndef __ROUTER_H_
#define __ROUTER_H_

#include <systemc.h>
#include "tlm.h"

SC_MODULE( Router )
    , public tlm::tlm_fw_transport_if<>
    , public tlm::tlm_bw_transport_if<>
{
    enum { TARGET_MAX = 256 };

    tlm::tlm_initiator_socket<32>  *i_socket[TARGET_MAX];
    tlm::tlm_target_socket<32>     t_socket;

    SC_HAS_PROCESS( Router );
    Router( sc_module_name name, const unsigned int target_um=4 );
    ~Router();

protected:
    // I/F functions
    // FW
    virtual void b_transport( tlm::tlm_generic_payload &trans, sc_time &time ) ;
    virtual tlm::tlm_sync_enum nb_transport_fw( tlm::tlm_generic_payload
            &trans, tlm::tlm_phase &phase, sc_time &time );
    virtual unsigned int transport_dbg( tlm::tlm_generic_payload &trans );
    virtual bool get_direct_mem_ptr( tlm::tlm_generic_payload &trans, tlm::tlm_dmi& dmi);
    // BW
    virtual tlm::tlm_sync_enum nb_transport_bw(
            tlm::tlm_generic_payload& trans, tlm::tlm_phase &phase, sc_time& time );
    virtual void invalidate_direct_mem_ptr( sc_dt::uint64 a, sc_dt::uint64 b);

    // functions
    unsigned int decode_address( unsigned int address, unsigned int& masked_address );
    sc_dt::uint64 compose_address( unsigned int target_nr, sc_dt::uint64 address);

private:
    const unsigned int targetNum;

};

#endif /* __ROUTER_H_ */
